__all__ = ['LEAGUE_LISTS']

LEAGUE_LISTS = [
    {'caption': 'European Championships France ', 'league': 'EC'},
    {'caption': 'Premier League', 'league': 'PL'},
    {'caption': 'Championship', 'league': 'ELC'},
    {'caption': 'League One', 'league': 'EL1'},
    {'caption': 'FA-Cup', 'league': 'FAC'},
    {'caption': '1. Bundesliga', 'league': 'BL1'},
    {'caption': '2. Bundesliga', 'league': 'BL2'},
    {'caption': 'DFB-Pokal', 'league': 'DFB'},
    {'caption': 'Eredivisie', 'league': 'DED'},
    {'caption': 'Ligue 1', 'league': 'FL1'},
    {'caption': 'Ligue 2', 'league': 'FL2'},
    {'caption': 'Primera Division', 'league': 'PD'},
    {'caption': 'Liga Adelante', 'league': 'SD'},
    {'caption': 'Serie A', 'league': 'SA'},
    {'caption': 'Primeira Liga', 'league': 'PPL'},
    {'caption': 'Champions League', 'league': 'CL'},
    {'caption': 'Serie B', 'league': 'SB'},
    {'caption': 'English National League', 'league': 'ENL'},
    {'caption': 'League Two', 'league': 'EL2'}
]



import requests
import click
from .exceptions import APIErrorException
from soccer.data import LEAGUE_LISTS, urls
import json

__all__ = ['RequestHandler']

class RequestHandler(object):

    BASE_URL = urls['base']
    SEASON_URL = urls['season']

    def __init__(self, headers, league_ids, team_names, writer):
        self.headers = headers
        self.league_ids = league_ids
        self.team_names = team_names
        self.writer = writer

    def _get(self, url):
        """Handles api.football-data.org requests"""
        req = requests.get(RequestHandler.BASE_URL + url, headers=self.headers)
        if req.status_code == requests.codes.ok:
            return req

        if req.status_code == requests.codes.bad:
            raise APIErrorException('Invalid request. Check parameters.')

        if req.status_code == requests.codes.forbidden:
            raise APIErrorException('This resource is restricted')

        if req.status_code == requests.codes.not_found:
            raise APIErrorException('This resource does not exist. Check parameters')

        if req.status_code == requests.codes.too_many_requests:
            raise APIErrorException('You have exceeded your allowed requests per minute/day')

    def _get_by_season(self, url):
        """Handles api.football-data.org requests"""
        req = requests.get(RequestHandler.SEASON_URL + url, headers=self.headers)

        if req.status_code == requests.codes.ok:
            return req

        if req.status_code == requests.codes.bad:
            raise APIErrorException('Invalid request. Check parameters.')

        if req.status_code == requests.codes.forbidden:
            raise APIErrorException('This resource is restricted')

        if req.status_code == requests.codes.not_found:
            raise APIErrorException('This resource does not exist. Check parameters')

        if req.status_code == requests.codes.too_many_requests:
            raise APIErrorException('You have exceeded your allowed requests per minute/day')


    def get_league(self, league):
        league_id = self.league_ids[league]
        try:
            req = self._get('competitions/{id}'.format(id=league_id))
            self.writer.league(req.json())
        except APIErrorException:
            click.secho("No information availble for {league}.".format(league=league),
                        fg="red", bold=True)

    def get_leagues(self):
        try:
            req = self._get('competitions')
            self.writer.leagues(req.json())
        except APIErrorException:
            click.secho("No information.",fg="red", bold=True)

    def get_leagues_by_season(self, season):
        try:
            req = self._get_by_season('soccerseasons?season={season}'
                            .format(season=season))
            self.writer.leagues_by_season(req.json())
        except APIErrorException:
            click.secho("No information availble for {season}."
                        .format(season=season), fg="red", bold=True)

    def get_standings(self, league):
        """Queries the API and gets the standings for a particular league"""
        league_id = self.league_ids[league]
        try:
            req = self._get('competitions/{id}/leagueTable'.format(id=league_id))
            self.writer.standings(req.json(), league)
        except APIErrorException:
            click.secho("No standings availble for {league}."
                    .format(league=league), fg="red", bold=True)

    def get_standings_by_matchday(self, league, md):
        """Queries the API and gets the standings for a particular league"""
        league_id = self.league_ids[league]
        try:
            req = self._get('competitions/{id}/leagueTable?matchday={md}'
                            .format(id=league_id,md=md))
            self.writer.standings(req.json(), league)
        except APIErrorException:
            click.secho("No standings availble for {league}."
                    .format(league=league),fg="red", bold=True)

    def get_teams(self, league):
        league_id = self.league_ids[league]
        try:
            req = self._get('competitions/{id}/teams'.format(id=league_id))
            self.writer.teams(req.json())
        except APIErrorException:
            click.secho("No teams availble for {league}."
                    .format(league=league), fg="red", bold=True)

    def get_players(self, team):
        team_id = self.team_names[team]
        try:
            req = self._get('teams/{id}/players'.format(id=team_id))
            self.writer.players(req.json(), team)
        except APIErrorException:
            click.secho("No players availble for {team}.".format(team=team),
                        fg="red", bold=True)

    def get_fixtures(self):
        try:
            req = self._get('fixtures')
            self.writer.fixtures(req.json())
        except APIErrorException:
            click.secho("No fixtures availble.", fg="red", bold=True)

    def get_fixtures_by_tf(self, tf):
        try:
            req = self._get('fixtures?timeFrame={tf}'.format(tf=tf))
            self.writer.fixtures(req.json())
        except APIErrorException:
            click.secho("No fixtures availble.", fg="red", bold=True)

    def get_fixtures_by_league(self, league):
        league_id = self.league_ids[league]
        try:
            req = self._get('competitions/{id}/fixtures'.format(id=league_id))
            self.writer.fixtures(req.json())
        except APIErrorException:
            click.secho("No teams availble for {league}."
                    .format(league=league), fg="red", bold=True)

    def get_fixtures_by_md_league(self, md, league):
        league_id = self.league_ids[league]
        try:
            req = self._get('competitions/{id}/fixtures?matchday={md}'
                            .format(id=league_id, md=md))
            self.writer.fixtures(req.json())
        except APIErrorException:
            click.secho("No fixtures availble.", fg="red", bold=True)


    def get_fixtures_by_tf_league(self, tf, league):
        league_id = self.league_ids[league]
        try:
            req = self._get('competitions/{id}/fixtures?timeFrame={tf}'
                            .format(id=league_id,tf=tf))
            self.writer.fixtures(req.json())
        except APIErrorException:
            click.secho("No fixtures availble.", fg="red", bold=True)

    def get_fixtures_by_md_tf_league(self, md, tf, league):
        league_id = self.league_ids[league]
        try:
            req = self._get('competitions/{id}/fixtures?matchday={md}&timeFrame={tf}'
                            .format(id=league_id, md=md, tf=tf))
            self.writer.fixtures(req.json())
        except APIErrorException:
            click.secho("No fixtures availble.", fg="red", bold=True)

    def get_fixtures_by_team(self, team):
        team_id = self.team_names[team]
        try:
            req = self._get('teams/{id}/fixtures'.format(id=team_id))
            self.writer.fixtures(req.json())
        except APIErrorException:
            click.secho("No fixtures availble for {team}."
                    .format(team=team), fg="red", bold=True)

    def get_fixtures_by_id(self, id, h2h):
        try:
            req = self._get('fixtures/{id}?head2head={h2h}'.format(id=id, h2h=h2h))
            self.writer.fixtures_by_id(req.json())
        except APIErrorException:
            click.secho("No fixtures availble.", fg="red", bold=True)

    def get_lists(self):
        try:
            self.writer.lists(LEAGUE_LISTS)
        except APIErrorException:
            click.secho("No leagues availble.", fg="red", bold=True)

    def jsonify(self,data):
        return json.dumps(data, sort_keys=True, indent=4, ensure_ascii=False)



from abc import ABCMeta, abstractmethod
from itertools import groupby

from soccer.data import LEAGUE_IDS

__all__ = ['BaseWriter']

class BaseWriter(object):
    __metaclass__ = ABCMeta

    def __init__(self, output_file):
        self.output_filename = output_file

    @abstractmethod
    def live_scores(self, live_scores):
        pass

    @abstractmethod
    def team_scores(self, team_scores, time):
        pass

    @abstractmethod
    def team_players(self, team):
        pass

    @abstractmethod
    def standings(self, league_table, league):
        pass

    @abstractmethod
    def league_scores(self, total_data, time):
        pass

    def supported_leagues(self, total_data):
        """Filters out scores of unsupported leagues"""
        supported_leagues = {val: key for key, val in LEAGUE_IDS.items()}
        get_league_id = lambda x: int(x["_links"]["soccerseason"]["href"].split("/")[-1])
        fixtures = (fixture for fixture in total_data["fixtures"]
                    if get_league_id(fixture) in supported_leagues)

        # Sort the scores by league to make it easier to read
        fixtures = sorted(fixtures, key=get_league_id)
        for league, scores in groupby(fixtures, key=get_league_id):
            league = supported_leagues[league]
            for score in scores:
                yield league, score



from .base import BaseWriter
from .writers import Stdout, Fstdout, Json, Fjson
from .team_writers import Gjson, Gstdout
import io

def get_writer(output_format='stdout', output_file=None):
    return globals()[output_format.capitalize()](output_file)



from .leagueids import LEAGUE_IDS
from .leaguelists import LEAGUE_LISTS
from .leaguenames import LEAGUE_NAMES
from .leagueproperties import LEAGUE_PROPERTIES
from .teamnames import TEAM_NAMES
from .config import config
from .urls import urls



from . import BaseWriter
import click
import json

__all__ = ['Gstdout', 'Gjson']

class Gstdout(BaseWriter):
    def __init__(self, output_file):
        self.colors = dict(
            A="green",
            B="yellow",
            C="blue",
            D="red",
            E="green",
            F="yellow",
            G="blue",
            H="red",
        )
    def standings(self, league_table, league):
        """ Prints the league standings in a pretty way """
        click.secho("-"*100, fg="green", bold=True)
        click.secho("%-2s  %-5s %-25s %-10s %-10s %-11s %-11s %-10s" %
                    ("GP", "POS", "CLUB", "PLAYED",  "GOALS", "GOAL LOSS", "GOAL DIFF","POINTS"))
        click.secho("-"*100, fg="green", bold=True)

        for key in sorted(league_table["standings"].keys()):
            for team in league_table["standings"][key]:
                if team["goalDifference"] >= 0:
                    team["goalDifference"] = ' ' + str(team["goalDifference"])
                team_str = (u"{group:<3} {rank:<5} {team:<25} "
                            u"{playedGames:<10} {goals:<10} {goalsAgainst:<10}"
                            u" {goalDifference:<12} {points}").format(**team)
                click.secho(team_str, bold=True, fg=self.colors[team['group']])

        click.secho("-"*100, fg="green", bold=True)

class Gjson(BaseWriter):
    def __init__(self, output_file):
        super(Gjson, self).__init__(output_file)

    def generate_output(self, result):
        click.echo(json.dumps(result, indent=4, separators=(',', ': '),
                   ensure_ascii=False))

    def standings(self, league_table, league):
        self.generate_output(league_table)



from soccer.data import LEAGUE_PROPERTIES, LEAGUE_NAMES
from . import BaseWriter

import click
import json
import io

__all__ = ['Stdout', 'Fstdout', 'Fjson', 'Json']

class Stdout(BaseWriter):
    def __init__(self, output_file):
        enums = dict(
            CL_POSITION="green",
            EL_POSITION="yellow",
            RL_POSITION="red",
            POSITION="blue",
            TEXT="yellow",
        )
        self.colors = type('Enum', (), enums)


    def league(self, league_table):
        click.secho("-"*100, fg="green", bold=True)
        click.secho("%-35s %-8s %-8s %-8s %-8s %-8s %-8s" %
                    ("CAPTION", "CM", "LEAGUE", "GAMES","MD", "TEAMS", "YEAR"))
        click.secho("-"*100, fg="green", bold=True)
        league_str = (u"{caption:<35} {currentMatchday:<8} {league:<9}"
                      u"{numberOfGames:<8} {numberOfMatchdays:<9}"
                      u"{numberOfTeams:<8} {year}").format(**league_table)
        click.secho(league_str, bold=True, fg=self.colors.TEXT)
        click.secho("-"*100, fg="green", bold=True)

    def leagues(self, league_table):
        click.secho("-"*100, fg="green", bold=True)
        click.secho("%-35s %-8s %-8s %-8s %-8s %-8s %-8s" %
                    ("CAPTION", "CM", "LEAGUE", "GAMES","MD", "TEAMS", "YEAR"))
        click.secho("-"*100, fg="green", bold=True)

        for league in league_table:
            league_str = (u"{caption:<35} {currentMatchday:<8} {league:<9}"
                          u"{numberOfGames:<8} {numberOfMatchdays:<9}"
                          u"{numberOfTeams:<8} {year}").format(**league)
            click.secho(league_str, bold=True, fg=self.colors.TEXT)

        click.secho("-"*100, fg="green", bold=True)


    def leagues_by_season(self, league_table):
        click.secho("-"*80, fg="green", bold=True)
        click.secho("%-35s %-8s %-8s %-8s %-8s" %
                    ("CAPTION", "LEAGUE", "GAMES", "TEAMS", "YEAR"))
        click.secho("-"*80, fg="green", bold=True)

        for league in league_table:
            league_str = (u"{caption:<35} {league:<9} {numberOfGames:<8}"
                          u"{numberOfTeams:<8} {year}").format(**league)
            click.secho(league_str, bold=True, fg=self.colors.TEXT)
        click.secho("-"*80, fg="green", bold=True)

    def standings(self, league_table, league):
        """ Prints the league standings in a pretty way """
        click.secho("-"*80, fg="green", bold=True)
        click.secho("%-6s  %-30s    %-10s    %-10s    %-10s" %
                    ("POS", "CLUB", "PLAYED", "GOAL DIFF", "POINTS"))
        click.secho("-"*80, fg="green", bold=True)

        for team in league_table["standing"]:
            if team["goalDifference"] >= 0:
                team["goalDifference"] = ' ' + str(team["goalDifference"])

            cl_upper, cl_lower = LEAGUE_PROPERTIES[league]['cl']
            el_upper, el_lower = LEAGUE_PROPERTIES[league]['el']
            rl_upper, rl_lower = LEAGUE_PROPERTIES[league]['rl']

            team_str = (u"{position:<7} {teamName:<33} {playedGames:<12}"
                        u" {goalDifference:<14} {points}").format(**team)
            if cl_upper <= team["position"] <= cl_lower:
                click.secho(team_str, bold=True, fg=self.colors.CL_POSITION)
            elif el_upper <= team["position"] <= el_lower:
                click.secho(team_str, fg=self.colors.EL_POSITION)
            elif rl_upper <= team["position"] <= rl_lower:
                click.secho(team_str, fg=self.colors.RL_POSITION)
            else:
                click.secho(team_str, fg=self.colors.POSITION)
        click.secho("-"*80, fg="green", bold=True)

    def teams(self, league_table):
        click.secho("-"*80, fg="green", bold=True)
        click.secho("%-28s %-20s %-10s %-20s" %
                    ("NAME", "SHORT NAME", "CODE", "MARKET VALUE"))
        click.secho("-"*80, fg="green", bold=True)
        for team in league_table['teams']:
            self.toNone(team)
            team_str = (u"{name:<28} {shortName:<20} {code:<10}"
                        u"{squadMarketValue}").format(**team)
            click.secho(team_str, bold=True, fg=self.colors.TEXT)
        click.secho("-"*80, fg="green", bold=True)


    def players(self, league_table, team):
        click.secho("-"*120, fg="green", bold=True)
        click.secho("%-27s %-10s %-21s %-15s %-25s %-15s" %
                    ("NAME", "NUMBER", "POSITION","BIRTH",
                    "NATIONALITY", "CONTRACT UNTIL"))
        click.secho("-"*120, fg="green", bold=True)

        for player in league_table['players']:
            self.toNone(player)
            player_str = (u"{name:<27} {jerseyNumber:<10} {position:<22}"
                        u"{dateOfBirth:<15} {nationality:<25} {contractUntil}").format(**player)
            click.secho(player_str, bold=True, fg=self.colors.TEXT)
        click.secho("-"*120, fg="green", bold=True)

    def fixtures(self, league_table):
        click.secho("-"*120, fg="green", bold=True)
        click.secho("%-27s %-27s %-6s %-6s %-22s %-10s %-8s %-6s" %
                    ("HOME TEAM", "AWAY TEAM", "RES","MD",
                    "DATE", "STATUS", "ID", "LEAGUE"))
        click.secho("-"*120, fg="green", bold=True)

        for fixture in league_table['fixtures']:
            fixture['league'] = LEAGUE_NAMES[fixture['competitionId']]
            if fixture['result']['goalsAwayTeam'] is None:
                fixture['result'] = 'None'
            else:
                fixture['result'] = '{0}:{1}'.format(fixture['result']['goalsHomeTeam'], fixture['result']['goalsAwayTeam'])
            fixture_str= (u"{homeTeamName:<27} {awayTeamName:<27} "
                        u"{result:<6} {matchday:<6} {date:<22} "
                        u"{status:<10} {id:<8} {league}").format(**fixture)
            click.secho(fixture_str, bold=True, fg=self.colors.TEXT)
        click.secho("-"*120, fg="green", bold=True)

    def fixtures_by_id(self, league_table):
        click.secho("-"*120, fg="green", bold=True)
        click.secho("%-27s %-27s %-6s %-6s %-22s %-10s %-8s %-6s" %
                    ("HOME TEAM", "AWAY TEAM", "RES","MD",
                    "DATE", "STATUS", "ID", "LEAGUE"))
        click.secho("-"*120, fg="green", bold=True)

        for fixture in league_table['head2head']['fixtures']:
            fixture['league'] = LEAGUE_NAMES[fixture['competitionId']]
            if fixture['result']['goalsAwayTeam'] is None:
                fixture['result'] = 'None'
            else:
                fixture['result'] = '{0}:{1}'.format(
                        fixture['result']['goalsHomeTeam'], fixture['result']['goalsAwayTeam'])

            if fixture['status'] is None:
                fixture['status'] = 'FINISHED'

            fixture_str = (u"{homeTeamName:<27} {awayTeamName:<27} "
                        u"{result:<6} {matchday:<6} {date:<22} "
                        u"{status:<10} {id:<8} {league}").format(**fixture)
            click.secho(fixture_str, bold=True, fg=self.colors.TEXT)
        click.secho("-"*120, fg="green", bold=True)

        HOME = league_table['fixture']['homeTeamName']
        AWAY = league_table['fixture']['awayTeamName']
        HW = league_table['head2head']['homeTeamWins']
        AW = league_table['head2head']['awayTeamWins']
        COUNT = league_table['head2head']['count']

        league_table['head2head']['homeWins'] = round(HW / COUNT, 2)
        league_table['head2head']['awayWins'] = round(AW / COUNT, 2)

        click.secho("%-30s %-29s %-6s %-6s %-10s %-10s %-12s %-12s" %
                    (HOME + " WINS", AWAY + " WINS","DRAWS", "COUNT",
                    "HOME WINS", "AWAY WINS", "START", "END"))

        result_str = (u"{homeTeamWins:<30} {awayTeamWins:<30}"
            u"{draws:<6} {count:<6} {homeWins:<11}"
            u"{awayWins:<10} {timeFrameStart:<13}"
            u"{timeFrameEnd}").format(**league_table['head2head'])

        click.secho(result_str, bold=True, fg=self.colors.TEXT)
        click.secho("-"*120, fg="green", bold=True)

    def lists(self, league_table):
        click.secho("-"*45, fg="green", bold=True)
        click.secho("%-10s %-20s" %("LEAGUE", "CAPTION"))
        click.secho("-"*45, fg="green", bold=True)

        for _list in league_table:
            list_str= (u"{league:<10} {caption}").format(**_list)
            click.secho(list_str, bold=True, fg=self.colors.TEXT)
        click.secho("-"*45, fg="green", bold=True)

    def toNone(self, lists):
        for key in lists:
            if lists[key] is None:
                lists[key] = 'None'

class Fstdout(BaseWriter):
    def __init__(self,output_file):

        enums = dict(
            CL_POSITION="green",
            EL_POSITION="yellow",
            RL_POSITION="red",
            POSITION="blue",
        )
        self.colors = type('Enum', (), enums)

    def standings(self, league_table, league):
        """ Prints the league standings in a pretty way """
        click.secho("-"*120, fg="green", bold=True)
        click.secho("%-4s %-25s %-9s %-10s %-10s %-9s %-10s %-11s %-11s %-7s" %
                    ("POS", "CLUB", "PLAYED", "WINS", "DRAWS", "LOSSES",
                            "GOALS", "GOAL LOSS", "GOAL DIFF", "POINTS"))
        click.secho("-"*120, fg="green", bold=True)

        for team in league_table["standing"]:
            if team["goalDifference"] >= 0:
                team["goalDifference"] = ' ' + str(team["goalDifference"])

            cl_upper, cl_lower = LEAGUE_PROPERTIES[league]['cl']
            el_upper, el_lower = LEAGUE_PROPERTIES[league]['el']
            rl_upper, rl_lower = LEAGUE_PROPERTIES[league]['rl']

            team_str = (u"{position:<4} {teamName:<25} {playedGames:<10}"
                        u"{wins:<10} {draws:<10} {losses:<10}"
                        u"{goals:<10} {goalsAgainst:<10}"
                        u" {goalDifference:<12} {points}").format(**team)
            if cl_upper <= team["position"] <= cl_lower:
                click.secho(team_str, bold=True, fg=self.colors.CL_POSITION)
            elif el_upper <= team["position"] <= el_lower:
                click.secho(team_str, fg=self.colors.EL_POSITION)
            elif rl_upper <= team["position"] <= rl_lower:
                click.secho(team_str, fg=self.colors.RL_POSITION)
            else:
                click.secho(team_str, fg=self.colors.POSITION)
        click.secho("-"*120, fg="green", bold=True)

class Fjson(BaseWriter):
    def __init__(self, output_file):
        super(Fjson, self).__init__(output_file)

    def generate_output(self, result):
        if not self.output_filename:
            click.echo(json.dumps(result, indent=4, separators=(',', ': '),
                       ensure_ascii=False))
        else:
            with io.open(self.output_filename, 'w', encoding='utf-8') as json_file:
                data = json.dumps(result, json_file, indent=4,
                                  separators=(',', ': '), ensure_ascii=False)
                json_file.write(data)

    def standings(self, league_table, league):
        self.generate_output(league_table)

    def league(self, league_table):
        self.generate_output(league_table)

    def leagues(self, league_table):
        self.generate_output(league_table)

    def teams(self, league_table):
        self.generate_output(league_table)

    def players(self, league_table, team):
        self.generate_output(league_table)

    def fixtures(self, league_table):
        self.generate_output(league_table)

    def fixtures_by_id(self, league_table):
        self.generate_output(league_table)

    def leagues_by_season(self, league_table):
        self.generate_output(league_table)

    def lists(self, league_table):
        self.generate_output(league_table)


class Json(BaseWriter):
    def __init__(self, output_file):
        super(Json, self).__init__(output_file)

    def generate_output(self, result):
        if not self.output_filename:
            click.echo(json.dumps(result, indent=4, separators=(',', ': '),
                       ensure_ascii=False))
        else:
            with io.open(self.output_filename, 'w', encoding='utf-8') as json_file:
                data = json.dumps(result, json_file, indent=4,
                                  separators=(',', ': '), ensure_ascii=False)
                json_file.write(data)

    def standings(self, league_table, league):
        """Store output of league standings to a JSON file"""
        data = []
        for team in league_table['standing']:
            item = {'position': team['position'],
                    'teamName': team['teamName'],
                    'playedGames': team['playedGames'],
                    'goalsFor': team['goals'],
                    'goalsAgainst': team['goalsAgainst'],
                    'goalDifference': team['goalDifference'],
                    'points': team['points']}
            data.append(item)
        self.generate_output({'standings': data})

    def league(self, league_table):
        """Store output of league standings to a JSON file"""

        self.generate_output(league_table)

    def leagues(self, league_table):
        """Store output of league standings to a JSON file"""
        for league in league_table:
            del league['_links']
        self.generate_output(league_table)

    def teams(self, league_table):
        del league_table['_links']
        for team in league_table['teams']:
            del team['_links']
        self.generate_output(league_table)

    def players(self, league_table, team):
        del league_table['_links']
        self.generate_output(league_table)

    def fixtures(self, league_table):
        for fixture in league_table['fixtures']:
            del fixture['odds']
        self.generate_output(league_table)

    def leagues_by_season(self, league_table):
        for league in league_table:
            del league['_links']
        self.generate_output(league_table)

    def lists(self, league_table):
        self.generate_output(league_table)



from .exceptions import APIErrorException, IncorrectParametersException
from .request_handler import RequestHandler
from .main import main
from .data import config, urls, TEAM_NAMES, LEAGUE_IDS, LEAGUE_LISTS, LEAGUE_NAMES, LEAGUE_PROPERTIES
from .writers import get_writer

__author__ = 'RayYu03'
__version__ = '0.0.3'



import os
import sys
import json
import click

from soccer.data import LEAGUE_IDS, TEAM_NAMES, config, urls
from soccer.writers import get_writer
from .exceptions import IncorrectParametersException
from .request_handler import RequestHandler

__all__ = ['main']

key = os.environ.get('SOCCER_CLI_API_TOKEN')  or config['key']

@click.command()
@click.option('--apikey', default=key,
              help="API key to use.")
@click.option('--standings', is_flag=True,
              help="Standings for a particular league.")
@click.option('--lists', is_flag=True,
              help="Lists of all league.")
@click.option('--teams', is_flag=True,
              help="Teams for a particular league.")
@click.option('--team', type=click.Choice(TEAM_NAMES.keys()),
              help=("Choose a particular team."))
@click.option('--players', is_flag=True,
              help="Shows players for a particular team.")
@click.option('--fixtures', is_flag=True,
              help="Shows fixtures.")
@click.option('--id', '-id', type=int,
              help=("Choose the Matchday for team's fixtures."))
@click.option('--h2h', '-h2h', type=int, default=10,
              help=("Choose the numbers of head to head for recently fixtures."))
@click.option('--md', '-md', type=int,
              help=("Choose the Matchday for team's fixtures."))
@click.option('--tf', '-tf', type=int,
              help=("Choose a particular day for team's fixtures. \
                    Positive number means future fixtures, \
                    negative number means previous fixtures"))
@click.option('--league', '-league', type=click.Choice(LEAGUE_IDS.keys()),
              help=("Choose a particular league."))
@click.option('--competition', '-competition',is_flag=True,
              help=("Shows information of the all leagues."))
@click.option('--season', '-season', type=int,
              help=("Choose the season for competition."))
@click.option('--stdout', 'output_format', flag_value='stdout', default=True,
              help="Print to stdout.")
@click.option('--fs', 'output_format', flag_value='fstdout',
              help="Print to stdout for full standings table, except group game.")
@click.option('--gs', 'output_format', flag_value='gstdout',
              help='Print to stdout for the group game of the league.')
@click.option('--json', 'output_format', flag_value='json',
              help='Output in JSON format, except the standings table of group game.')
@click.option('--fjson', 'output_format', flag_value='fjson',
              help='Output in JSON format of the full version.')
@click.option('--gjson', 'output_format', flag_value='gjson',
              help='Output in JSON format for the group game of the league.')
@click.option('-o', '--output-file', default=None,
              help="Save output to a file (only json option is provided).")

def main(league, lists, teams, team, players, fixtures, id, h2h, md, tf,
        competition, season, standings, output_format, output_file, apikey):

    if not key:
        click.secho('No API Token detected. \n'
                    'Please visit {0} and get an API Token, '
                    'which will be used by Soccer '
                    'to get access to the data.'
                    .format(urls['web']), fg="red", bold=True)
        sys.exit(1)

    if fixtures:
        headers = {'X-Auth-Token': apikey, 'X-Response-Control': 'minified'}
    else:
        headers = {'X-Auth-Token': apikey}

    try:
        if output_format in ['stdout', 'fstdout', 'gstdout'] and output_file:
            raise IncorrectParametersException('Printing output to stdout and '
                                               'saving to a file are mutually exclusive')
        writer = get_writer(output_format, output_file)
        rh = RequestHandler(headers, LEAGUE_IDS, TEAM_NAMES, writer)

        if lists:
            rh.get_lists()

        if standings:
            if not league:
                raise IncorrectParametersException('Please specify a league. '
                                                   'Example --standings --league=EPL')
            if league in ['CL', 'EC'] and output_format not in ['gstdout', 'gjson']:
                raise IncorrectParametersException('Standings for CL and EC '
                                                   'Champions League must use --gs(group stdout) '
                                                   'and --gjson(group json) as output format.')
            elif league not in ['CL', 'EC'] and output_format in ['gstdout', 'gjson']:
                    raise IncorrectParametersException('Output format gstdout and gjson '
                                                       'only for EC and CL.')
            if md:
                rh.get_standings_by_matchday(league, md)
            else:
                rh.get_standings(league)

        if teams:
            if not league:
                raise IncorrectParametersException('Please specify a league. '
                                                   'Example --teams --league=PD')
            rh.get_teams(league)

        if competition:
            if league:
                rh.get_league(league)
            elif season:
                rh.get_leagues_by_season(season)
            else:
                rh.get_leagues()

        if players:
            if not team:
                raise IncorrectParametersException('Please specify a league. '
                                                   'Example --players --team=FCB')
            rh.get_players(team)

        if fixtures:
            if tf and md and league:
                tf = 'n' + str(tf) if tf > 0 else 'p' + str(abs(tf))
                rh.get_fixtures_by_md_tf_league(md, tf, league)
            elif tf and league:
                tf = 'n' + str(tf) if tf > 0 else 'p' + str(abs(tf))
                rh.get_fixtures_by_tf_league(tf, league)
            elif md and league:
                rh.get_fixtures_by_md_league(md, league)
            elif tf:
                tf = 'n' + str(tf) if tf > 0 else 'p' + str(abs(tf))
                rh.get_fixtures_by_tf(tf)
            elif team:
                rh.get_fixtures_by_team(team)
            elif league:
                rh.get_fixtures_by_league(league)
            elif id:
                rh.get_fixtures_by_id(id, h2h)

            if not league and not team and not id and not tf and not md:
                rh.get_fixtures()

    except IncorrectParametersException as e:
        click.secho(str(e), fg="red", bold=True)

if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import re
import soccer

from setuptools import setup, find_packages

with open('README.rst', 'rb') as f_readme:
    readme = f_readme.read().decode('utf-8')

packages = find_packages(exclude=["*.tests", "*.tests.*", "tests.*", "tests"])

setup(
    name='pysoccer',
    version=soccer.__version__,
    keywords=['soccer', 'football', 'football-data'],
    description='Soccer data for programmers.',
    long_description=readme,
    author='RayYu03',
    author_email='shiliuhuasheng@gmail.com',
    license='MIT',
    url='https://github.com/RayYu03/pysoccer',
    install_requires=['requests','click'],
    packages=packages,
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Intended Audience :: Information Technology',
        'Intended Audience :: End Users/Desktop',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP',
        'Programming Language :: Python :: 3 :: Only',
    ],
    entry_points={
        'console_scripts': [
            'soccer = soccer.main:main'
        ]
    }
)
